﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
	
-------------------------------------------------------------------------------
-- default layout values

nUI_StatusLayout =
{
	name          = "",				-- the system object name to give this bar
	width         = 0,				-- how wide to draw the bar (int > 0)
	height        = 0,				-- how high to draw the bar (int > 0)
	
	Options =
	{
		bar_name  = "",				-- a text name that can overlay the bar (string)
		show_name = "none",			-- where to show the name on the bar: min|max|center|none
		cur_value = "center",		-- where to show the current value: min|max|center|outside|none
		max_value = "center",		-- where to show the maximum value: min|max|center|outside|none
		min_value = "none",			-- where to show the minimum value: min|max|center|outside|none
		percent   = "none",			-- where to show the percent of max value: min|max|center|outside|none
		grow      = "right",		-- which direction to grow the bar in: left|right
		wrap      = "",				-- if wraparound is enabled, a reference to the marker art file
	},
	
	Values =
	{
		min    = 0,					-- what is the minimum value for this bar (int >= 0)
		max    = 100,				-- what is the maximum value for this bar (int > 0)
		cur    = 0,					-- what is the current value of the bar (min <= cur <= max)
		offset = 0,					-- how much to offset the start of the bar from 0
	},
	
	Colors =
	{
		min = { r=1, g=0.8196079, b=0, a=1, },	-- what color is the bar at cur == 0
		mid = { r=1, g=0.8196079, b=0, a=1, },	-- what color is the bar at cur == (max-min)/2
		max = { r=1, g=0.8196079, b=0, a=1, },	-- what color is the bar at cur == max
		bg  = { r=0, g=0, b=0, a=1, },			-- what is the background color of the bar
		animation = "",							-- animation file to use or nil for no animation
		overlay   = "Interface\\AddOns\\nUI\\Bars\\Art\\nUI_BarOverlay1.blp",	-- overlay file to use or nil for no overlay
	},
}

nUI.Class.StatusLayout = nUI.Class:CreatePrototype();
local prototype        = nUI.Class.StatusLayout.prototype;

-------------------------------------------------------------------------------
-- this may look a little like a PITA, but it makes it possible to add new values
-- at any time without having to parse the existing user config and insert, etc.

function prototype:init( user_defined )

	if not user_defined then user_defined = {}; end
	if not user_defined.Options then user_defined.Options = {}; end
	if not user_defined.Values then user_defined.Values = {}; end
	if not user_defined.Colors then user_defined.Colors = {}; end
	if not user_defined.Colors.min then user_defined.Colors.min = {}; end
	if not user_defined.Colors.mid then user_defined.Colors.mid = {}; end
	if not user_defined.Colors.max then user_defined.Colors.max = {}; end
	if not user_defined.Colors.bg then user_defined.Colors.bg = {}; end
	
	self.name         = user_defined.name or nUI_StatusLayout.name;
	self.width        = tonumber( user_defined.width or nUI_StatusLayout.width );
	self.height       = tonumber( user_defined.height or nUI_StatusLayout.height );
	self.parent_frame = user_defined.parent_frame or nUI_StatusLayout.parent_frame;

	self.Options = 
	{
		bar_name      = user_defined.Options.bar_name or nUI_StatusLayout.Options.bar_name,
		show_name     = strlower( user_defined.Options.show_name or nUI_StatusLayout.Options.show_name ),
		cur_value     = strlower( user_defined.Options.cur_value or nUI_StatusLayout.Options.cur_value ),
		max_value     = strlower( user_defined.Options.max_value or nUI_StatusLayout.Options.max_value ),
		min_value     = strlower( user_defined.Options.min_value or nUI_StatusLayout.Options.min_value ),
		percent       = strlower( user_defined.Options.percent or nUI_StatusLayout.Options.percent ),
		grow          = strlower( user_defined.Options.grow or nUI_StatusLayout.Options.grow ),
	};
	
	self.Values = 
	{
		min           = tonumber( user_defined.Values.min or nUI_StatusLayout.Values.min ),
		max           = tonumber( user_defined.Values.max or nUI_StatusLayout.Values.max ),
		cur           = tonumber( user_defined.Values.cur or nUI_StatusLayout.Values.cur ),
		offset        = tonumber( user_defined.Values.offset or nUI_StatusLayout.Values.offset ),
	};
	
	self.Colors = 
	{
		min =
		{
			r         = tonumber( user_defined.Colors.min.r or nUI_StatusLayout.Colors.min.r ),
			g         = tonumber( user_defined.Colors.min.g or nUI_StatusLayout.Colors.min.g ),
			b         = tonumber( user_defined.Colors.min.b or nUI_StatusLayout.Colors.min.b ),
			a         = tonumber( user_defined.Colors.min.a or nUI_StatusLayout.Colors.min.a ),
		},
		mid =
		{
			r         = tonumber( user_defined.Colors.mid.r or nUI_StatusLayout.Colors.mid.r ),
			g         = tonumber( user_defined.Colors.mid.g or nUI_StatusLayout.Colors.mid.g ),
			b         = tonumber( user_defined.Colors.mid.b or nUI_StatusLayout.Colors.mid.b ),
			a         = tonumber( user_defined.Colors.mid.a or nUI_StatusLayout.Colors.mid.a ),
		},
		max =
		{
			r         = tonumber( user_defined.Colors. max.r or nUI_StatusLayout.Colors.max.r ),
			g         = tonumber( user_defined.Colors. max.g or nUI_StatusLayout.Colors.max.g ),
			b         = tonumber( user_defined.Colors. max.b or nUI_StatusLayout.Colors.max.b ),
			a         = tonumber( user_defined.Colors. max.a or nUI_StatusLayout.Colors.max.a ),
		},
		bg =
		{
			r         = tonumber( user_defined.Colors.bg.r or nUI_StatusLayout.Colors.bg.r ),
			g         = tonumber( user_defined.Colors.bg.g or nUI_StatusLayout.Colors.bg.g ),
			b         = tonumber( user_defined.Colors.bg.b or nUI_StatusLayout.Colors.bg.b ),
			a         = tonumber( user_defined.Colors.bg.a or nUI_StatusLayout.Colors.bg.a ),
		},
		animation     = user_defined.Colors.animation or nUI_StatusLayout.Colors.animation,
		overlay       = user_defined.Colors.overlay or nUI_StatusLayout.Colors.overlay,
	};
end
